unit ubiof;

// __TGL_RESIZE_WINDOW__        Resizable window
// __TGL_ALLOW_CLOSE__          window can be closed
// __TGL_CLOSE_ON_ESCAPE__      Close window when escape is pressed
// __TGL_WINDOWED__             Window or fullscreen
// __TGL_CENTER_WINDOW__        Place window in center of screen
// __TGL_DISABLE_SCREENSAVER__  Disable the screensaver
// __TGL_ICON__                 Show Icon

interface

procedure MainFunc;


implementation

uses
  mmsystem,
  windows,
  messages,
  opengl,
  tgl;

//{$DEFINE TRIANGLES}
//{$DEFINE SPHERES}
//{$DEFINE BIGSPHERES}
//{$DEFINE POINTS}
{$DEFINE LIGHTMAP}

{$IFDEF TRIANGLES}
const
  NRPOINTS = 80;
  NRLINES  = 30;
{$ENDIF}
{$IFDEF SPHERES}
const
  NRPOINTS = 80;
  NRLINES  = 30;
{$ENDIF}
{$IFDEF BIGSPHERES}
  {$DEFINE SPHERES}
const
  NRPOINTS = 20;
  NRLINES  = 5;
{$ENDIF}
{$IFDEF POINTS}
const
  NRPOINTS = 250;
  NRLINES  = 30;
{$ENDIF}
{$IFDEF LIGHTMAP}
const
  NRPOINTS = 150;
  NRLINES  = 20;
{$ENDIF}


// function was incorrectly implemented in opengl.pas
function  FixedgluBuild2DMipmaps(target: GLenum; components, width, height: GLint;
            format, atype: GLenum; data: Pointer): Integer; stdcall; external 'glu32.dll' name 'gluBuild2DMipmaps';


var
  screenwidth:     Integer;
  screenheight:    Integer;
{$IFDEF SPHERES}
  shapes:          GLUquadricObj;
{$ENDIF}  
{$ifdef __TGL_WINDOWED__}
  NrFrame:   Integer;
  startTime: double;
{$endif}

function FloatToStr(Value: Extended): string;
begin
  Result := '';
  if Value > 1000 then
    Result := Result + Char(Ord('0') + Trunc(Value/1000));
  if Value > 100 then begin
    Value := value - 1000 * Trunc(Value/1000);
    Result := Result + Char(Ord('0') + Trunc(Value/100));
  end;
  if Value > 10 then begin
    Value := value - 100 * Trunc(Value/100);
    Result := Result + Char(Ord('0') + Trunc(Value/10));
  end;
  Value := value - 10 * Trunc(Value/10);
  Result := Result + Char(Ord('0') + Trunc(Value)) + '.';
  Value := value - Trunc(Value);
  Value := Value * 10;
  Result := Result + Char(Ord('0') + Trunc(Value));
  Value := value - Trunc(Value);
  Value := Value * 10;
  Result := Result + Char(Ord('0') + Trunc(Value));
end;

procedure Horn(ribs: integer; bend, stack, twist, twisttrans: double; grow: double);
var
  i: integer;
begin
  for i:= 0 to ribs - 1 do begin
    glpushmatrix;
    glrotated(i * bend/ribs, 1,0,0);
    gltranslated(0,0, i * stack/ribs);
    gltranslated(twisttrans,0,0);
    glrotated(i * twist/ribs, 0, 0, 1);
    gltranslated(-twisttrans,0,0);

{$IFDEF TRIANGLES}
    glBegin(GL_Triangles);
    glcolor4d(0.9,0.8,0.5, 1);
    glVertex3d(2,2,0);
    glcolor4d(0.9,0.8,0.5,0.0);
    glVertex3d(0,2,0);
    glVertex3d(2,0,0);
    glEnd();
{$ENDIF}
{$IFDEF BIGSPHERES}
    gluQuadricDrawStyle(shapes, GLU_FILL);
    gluQuadricNormals(shapes, GLU_SMOOTH);
    gluSphere(shapes, 7 * Exp(i/ribs * Ln(grow)), 20, 10);
{$ELSE}
{$IFDEF SPHERES}
    gluQuadricDrawStyle(shapes, GLU_FILL);
    gluQuadricNormals(shapes, GLU_SMOOTH);
//    gluSphere(shapes, power(grow,i/ribs), 5, 5);
    gluSphere(shapes, Exp(i/ribs * Ln(grow)), 6, 4);
{$ENDIF}
{$ENDIF}
{$IFDEF LIGHTMAP}
    glBegin(GL_POINTS);
    glVertex3d(0,0,0);
    glEnd();
{$ENDIF}
{$IFDEF POINTS}
    glBegin(GL_POINTS);
    glVertex3d(0,0,0);
    glEnd();
{$ENDIF}
     glpopmatrix;
  end;
end;


procedure ResizeGL(Width, height: Integer);
begin
  screenWidth := width;
  screenheight := height;
  glViewport(0 , 0, Width, Height);
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity;
  gluPerspective(30, Width/Height, 100, 300);
end;

procedure InitGL;
const
  glfLightPosition:      Array[0..3] of GLfloat = (100.0, 100.0, 100.0, 0.0);
  glfFog          :      Array[0..3] of GLfloat = (0.0, 0.0, 0.3, 1.0);
//  SpecularMaterialColor: Array [0..3] of GLfloat = (0.7, 0.7, 0.7, 1.0);
  DiffuseLightColor:     Array [0..3] of GLfloat = (1, 0.8, 0.4, 1.0);
{$IFDEF LIGHTMAP}
var
  i,j: integer;
  x,y,r,d: double;
  texbuf: array[0..4095] of byte;
{$ENDIF}
begin
  glEnable(GL_DEPTH_TEST);

  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glEnable(gl_Blend);

{$IFDEF SPHERES}
  glEnable(GL_CULL_FACE);
  glCullFace(GL_BACK);
{$ENDIF}

{$IFDEF SPHERES}
  glLightfv(GL_LIGHT0, GL_POSITION, @glfLightPosition);
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
//  glEnable(GL_COLOR_MATERIAL);

  glLightfv(GL_LIGHT0, GL_DIFFUSE , @DiffuseLightColor);
//  glmaterialfv(GL_FRONT, GL_SPECULAR , @SpecularmaterialColor);
//  glmateriali(GL_FRONT, GL_SHININESS, 64);

{$ENDIF}

{$IFDEF POINTS}
  glHint (GL_POINT_SMOOTH_HINT, GL_NICEST);
  glPointSize(3);
  glEnable (GL_POINT_SMOOTH);
{$ENDIF}

{$IFDEF SPHERES}
(*
  glFogi (GL_FOG_MODE, GL_LINEAR);
  glHint (GL_FOG_HINT, GL_NICEST);
  glFogfv (GL_FOG_COLOR, @glfFog);
  glFogf(GL_FOG_START,150);
  glFogf(GL_FOG_End,550);
  glEnable(GL_FOG);
*)
{$ENDIF}
{$IFDEF POINTS}
  glFogi (GL_FOG_MODE, GL_LINEAR);
  glHint (GL_FOG_HINT, GL_NICEST);
  glFogfv (GL_FOG_COLOR, @glfFog);
  glFogf(GL_FOG_START,150);
  glFogf(GL_FOG_End,250);
  glEnable(GL_FOG);
{$ENDIF}

{$IFDEF SPHERES}
  shapes := gluNewQuadric();
{$ENDIF}

{$IFDEF LIGHTMAP}
  r := 32;
  for i := 0 to 63 do begin
    for j := 0 to 63 do begin
      x := abs(i - 32);
      y := abs(j - 32);
      d := sqrt(x * x + y * y);
      if d < r then begin
        d := 1 - (d/r);
        texbuf[i * 64 + j] := Round(255 * d * d);
      end else begin
        texbuf[i * 64 + j] := 0;
      end;
    end;
  end;

  FixedgluBuild2DMipmaps(GL_TEXTURE_2D, 1, 64, 64,
		    GL_Luminance,
		    GL_UNSIGNED_BYTE, @texbuf[0]);

  glTexParameteri(GL_TEXTURE_2D,
		  GL_TEXTURE_MIN_FILTER,
		  GL_LINEAR_MIPMAP_NEAREST);
  glTexParameteri(GL_TEXTURE_2D,
		  GL_TEXTURE_MAG_FILTER,
		  GL_LINEAR);
{$ENDIF}

{$IFDEF LIGHTMAP}
  // trick to fullscreen
  ShowWindow(wnd,SW_MAXIMIZE);
  PostMessage(wnd,WM_SIZE,0,0);
{$ENDIF}
end;

{$IFDEF LIGHTMAP}
procedure PaintGL;
{$ifdef __TGL_WINDOWED__}
var
  s: string;
  t: double;
{$endif}
var
  dt: double;
  i,j, offset: integer;
  fb_buffer: array[0 .. NRPOINTS * NRLINES * 4] of GLFloat;
  x,y,w: double;
begin
  dt := timegetTime * 0.001;

  glDisable(GL_DEPTH_TEST);
  glFeedbackBuffer(NRPOINTS * NRLINES * 4, GL_3D, Pointer(@fb_buffer[0]));
  glRenderMode(GL_FEEDBACK);

  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity;
  gluLookAt( 200, 0, 0,
             0, 0, 0,
             0, 0, 1);

  glPushMatrix();
  glRotated(90, 0, 1, 0);
  glRotated(10 * dt, 0, 0, 1);

  for i := 0 to NRLINES - 1 do begin
    glPushMatrix;
    glrotated(360 * i/NRLINES,0,0,1);
    glrotated(45,1,0,0);
    gltranslated(0,0,2);
    horn(NRPOINTS,
         315 * sin(dt * 0.237),
         45{ + 20 * sin(dt * 0.133)},
         1500 * sin(dt * 0.213),
         7.5 + 2.5 * sin(dt * 0.173),
         1);

    glPopMatrix;
  end;
  glPopMatrix();
  glPopMatrix();

  glRenderMode(GL_RENDER);

  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();
  glLoadIdentity();
  glScalef(2.0 / Screenwidth, 2.0 / ScreenHeight, 1.0);
  glTranslatef(0.5 * -Screenwidth, 0.5 * -ScreenHeight, 0.0);

  glClear(GL_COLOR_BUFFER_BIT);

  glEnable(GL_TEXTURE_2D);
  glBlendFunc(GL_ONE, GL_ONE);
  glEnable(GL_BLEND);

  glBegin(GL_QUADS);

  for j := 0 to NRPOINTS - 1 do begin
    glColor3f(0.25 - 0.15  * j/NRPOINTS, 0.15 + 0.10 * j/NRPOINTS, 0.06 + 0.14 * j/NRPOINTS);
    for i := 0 to NRLINES - 1 do begin

      offset := 4 * (i * NRPOINTS + j);
      x := fb_buffer[offset + 1];
      y := fb_buffer[offset + 2];

      w := 16;

      glTexCoord2f(0.0, 0.0);
      glVertex2f(x - w, y - w);

      glTexCoord2f(1.0, 0.0);
      glVertex2f(x + w, y - w);

      glTexCoord2f(1.0, 1.0);
      glVertex2f(x + w, y + w);

      glTexCoord2f(0.0, 1.0);
      glVertex2f(x - w, y + w);
    end;
  end;

  glEnd();

  glMatrixMode(GL_MODELVIEW);
  glPopMatrix();
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();

{$ifdef __TGL_WINDOWED__}
  Inc(NrFrame);
  t := (timeGetTime * 0.001) - StartTime;
  if t > 2 then begin
    s := 'BioF ' + FloatToStr(NrFrame/t) + ' fps';

    SendMessage(wnd, WM_SETTEXT, 0, Longint(Pchar(s)));

    startTime := timeGetTime * 0.001;
    NrFrame := 0;
  end;
{$endif}
end;

{$ELSE}
procedure PaintGL;
{$ifdef __TGL_WINDOWED__}
var
  s: string;
  t: double;
{$endif}
var
  dt: double;
  i: integer;
begin
  dt:= timegetTime * 0.001;

  glClearColor(0, 0, 0.3, 0);

  glClear(GL_DEPTH_BUFFER_BIT or GL_COLOR_BUFFER_BIT);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity;
  gluLookAt( 200, 00, 0,
             0, 0, 0,
             0, 0, 1);

  glPushMatrix;
  glRotated(90, 0, 1, 0);
  glRotated(10 * dt, 0, 0, 1);

  glcolor3d(0.9,0.7,0.5);

  glNewList(1, GL_COMPILE);
  horn(NRPOINTS,
       315 * sin(dt * 0.237),
       50 + 20 * sin(dt * 0.133),
       1500 * sin(dt * 0.213),
       7.5 + 2.5 * sin(dt * 0.173),
       0.6 + 0.1 * sin(dt * 0.317));
  glEndList;

  for i := 0 to NRLINES -1 do begin
    glPushMatrix;
    glrotated(360 * i/NRLINES,0,0,1);
    glrotated(45,1,0,0);
    gltranslated(0,0,2);
    glCallList(1);

    glPopMatrix;
  end;

  glPopMatrix;

{$ifdef __TGL_WINDOWED__}
  Inc(NrFrame);
  t := (timeGetTime * 0.001) - StartTime;
  if t > 2 then begin
    s := 'BioF ' + FloatToStr(NrFrame/t) + ' fps';

    SendMessage(wnd, WM_SETTEXT, 0, Longint(Pchar(s)));

    startTime := timeGetTime * 0.001;
    NrFrame := 0;
  end;
{$endif}
end;
{$ENDIF}


procedure CloseGL;
begin
{$IFDEF SPHERES}
  gluDeleteQuadric(shapes);
{$ENDIF}
end;

procedure MainFunc;
begin
{$ifdef __TGL_WINDOWED__}
  startTime := timeGetTime * 0.001;
  NrFrame := 0;
{$endif}

  screenWidth := 640;
  screenheight := 480;

  set_onglinit(InitGl);
  set_onglpaint(PaintGl);
  set_onglresize(ResizeGl);
  set_onglclose(CloseGl);

  gl_open('BioF', screenWidth, screenheight);
end;

end.

